"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getClassPuml = exports.getClass = exports.getPackagePuml = exports.getPackage = exports.convert = void 0;
const node_fs_1 = require("node:fs");
const promises_1 = require("node:fs/promises");
const node_path_1 = require("node:path");
const class_1 = require("./converter/class");
Object.defineProperty(exports, "getClassPuml", { enumerable: true, get: function () { return class_1.getClassPuml; } });
const convert_1 = require("./converter/convert");
Object.defineProperty(exports, "convert", { enumerable: true, get: function () { return convert_1.convert; } });
const package_1 = require("./converter/package");
Object.defineProperty(exports, "getPackagePuml", { enumerable: true, get: function () { return package_1.getPackagePuml; } });
const files_1 = require("./files");
const class_2 = require("./parser/class");
Object.defineProperty(exports, "getClass", { enumerable: true, get: function () { return class_2.getClass; } });
const package_2 = require("./parser/package");
Object.defineProperty(exports, "getPackage", { enumerable: true, get: function () { return package_2.getPackage; } });
const defaultOptions = {
    output: undefined,
    packages: undefined,
    linkbyfields: true,
    inheritance: true,
    format: 'puml',
};
const defaultHandler = {
    get(target, name) {
        return target.hasOwnProperty(name) ? target[name] : defaultOptions[name];
    },
};
/**
 * Converts Java Files to Plantuml and optionally saves them
 * @param path the root path of a Java project
 * @param options the conversion options
 * @returns the resulting plantuml or the output paths, if options.output was specified
 */
async function javatopuml(path, options) {
    options = new Proxy(options ?? defaultOptions, defaultHandler);
    const packages = options.packages?.length === 0 ? undefined : options.packages;
    if (options.output)
        return convertAndSave(path, packages, options);
    if (!packages || typeof packages === 'string') {
        const packagePath = await (0, files_1.findPackage)(path, packages);
        if (!packagePath)
            return;
        const javaPackage = await (0, package_2.getPackage)(packagePath, packages);
        const packagePuml = (0, convert_1.convert)(javaPackage, options);
        return packagePuml;
    }
    // multiple packages provided
    const results = [];
    for (const p of packages) {
        options.packages = p;
        const resultPuml = javatopuml(path, options);
        if (typeof resultPuml !== 'string')
            continue;
        results.push(resultPuml);
    }
}
exports.default = javatopuml;
/**
 * Converts java files and saves the resulting PlantUML code.
 * @param path the root of a java project
 * @param packages [the packages that should be converted]
 * @param options [an options object]
 * @returns the save locations of the generated PlantUML code
 */
async function convertAndSave(path, packages, options) {
    if (!options)
        options = defaultOptions;
    // no packages or one package provided
    if (!packages || typeof packages === 'string') {
        const packagePath = await (0, files_1.findPackage)(path, packages);
        if (!packagePath)
            return;
        const javaPackage = await (0, package_2.getPackage)(packagePath, packages);
        const packagePuml = (0, convert_1.convert)(javaPackage, options);
        const resultPath = (0, node_path_1.join)(options.output ?? '.', `${javaPackage.name}.${options.format}`);
        const resultDir = (0, node_path_1.parse)(resultPath).dir;
        if (!(0, node_fs_1.existsSync)(resultDir) && !(await (0, promises_1.mkdir)(resultDir, { recursive: true }))) {
            return;
        }
        await (0, promises_1.writeFile)(resultPath, packagePuml);
        return resultPath;
    }
    // multiple packages provided
    const resultPaths = [];
    for (const p of packages) {
        const resultPath = await convertAndSave(path, p, options);
        if (typeof resultPath !== 'string')
            continue;
        resultPaths.push(resultPath);
    }
    return resultPaths;
}
